# -*- coding: utf-8 -*-

"""
Tests for decorators

The decorator functions provide some safety around the parameters provided to API calls
"""
import pytest
from smartystreets.client import validate_args, truncate_args


def test_truncate_args(mocker):

    selfarg = mocker.MagicMock()
    myargs = [i for i in range(104)]

    def somefunc(myself, args):
        return len(args)

    selfarg.truncate_addresses = True
    assert truncate_args(somefunc)(selfarg, myargs) == 100

    selfarg.truncate_addresses = False
    modified_func = truncate_args(somefunc)
    with pytest.raises(ValueError):
        modified_func(selfarg, myargs)


def test_validate_args(mocker):
    def somefunc(myself, args):
        return len(args)

    selfarg = mocker.MagicMock()

    myargs = ["1", "2", "3"]
    assert validate_args(somefunc)(selfarg, myargs) == 3

    myargs = [1, 2, 3]
    modified_func = validate_args(somefunc)
    with pytest.raises(TypeError):
        modified_func(selfarg, myargs)

    myargs = [{}, "?"]
    modified_func = validate_args(somefunc)
    with pytest.raises(TypeError):
        modified_func(selfarg, myargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_smartystreets
----------------------------------

Tests for `smartystreets` module.
"""

import pytest
from smartystreets.data import Address, AddressCollection


class TestAddress:
    def test_empty(self):
        addr = Address([])
        assert addr.location is None

    def test_location(self):
        addr = Address({"metadata": {"latitude": 12, "longitude": 13}})
        assert addr.location == (12, 13)

    def test_id(self):
        addr = Address({"input_id": "test1"})
        assert addr.id == "test1"

    def test_index(self):
        addr = Address({"input_id": "test1", "input_index": 4})
        assert addr.index == 4


class TestAddressCollection:
    def test_get_by_id(self):
        """Ensure an address can be found by input_id"""
        collection = AddressCollection(
            [
                {"input_index": 3, "input_id": "A"},
                {"input_index": 5, "input_id": "h"},
                {"input_index": 8, "input_id": "X"},
            ]
        )
        assert collection.get("A").index == 3
        assert collection.get("h").index == 5
        assert collection.get("X").index == 8

        with pytest.raises(KeyError):
            collection.get_index("kj")

    def test_get_by_index(self):
        """Ensure an address can be found by input index"""
        collection = AddressCollection(
            [
                {"input_index": 3, "input_id": "A"},
                {"input_index": 5, "input_id": "h"},
                {"input_index": 8, "input_id": "X"},
            ]
        )
        assert collection.get_index(3).id == "A"
        assert collection.get_index(5).id == "h"
        assert collection.get_index(8).id == "X"

        with pytest.raises(KeyError):
            collection.get_index(10)

    def test_get_by_list_index(self):
        """Normal list indexing is unaffected"""
        collection = AddressCollection(
            [
                {"input_index": 0, "input_id": "A"},
                {"input_index": 1, "input_id": "h"},
                {"input_index": 2, "input_id": "X"},
            ]
        )
        assert collection[0].id == "A"
        assert collection[1].id == "h"
        assert collection[2].id == "X"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_smartystreets
----------------------------------

Tests for `smartystreets` module.
"""

import pytest
import httpx

from smartystreets.client import Client
from smartystreets import data
from smartystreets import exceptions


@pytest.fixture
def smarty_client():
    yield Client(auth_id="blah", auth_token="blibbidy")


class TestClient:
    @pytest.mark.respx(base_url="https://api.smartystreets.com")
    def test_input_error(self, smarty_client, respx_mock):
        respx_mock.post("/street-address", data="").mock(
            return_value=httpx.Response(400)
        )
        with pytest.raises(exceptions.SmartyStreetsInputError):
            smarty_client.street_addresses([{}, {}])

    @pytest.mark.respx(base_url="https://api.smartystreets.com")
    def test_auth_error(self, smarty_client, respx_mock):
        respx_mock.post("/street-address", data="").mock(
            return_value=httpx.Response(401)
        )
        with pytest.raises(exceptions.SmartyStreetsAuthError):
            smarty_client.street_addresses([{}, {}])

    @pytest.mark.respx(base_url="https://api.smartystreets.com")
    def test_payment_error(self, smarty_client, respx_mock):
        respx_mock.post("/street-address", data="").mock(
            return_value=httpx.Response(402)
        )
        with pytest.raises(exceptions.SmartyStreetsPaymentError):
            smarty_client.street_addresses([{}, {}])

    @pytest.mark.respx(base_url="https://api.smartystreets.com")
    def test_server_error(self, smarty_client, respx_mock):
        respx_mock.post("/street-address", data="").mock(
            return_value=httpx.Response(500)
        )
        with pytest.raises(exceptions.SmartyStreetsServerError):
            smarty_client.street_addresses([{}, {}])

    @pytest.mark.respx(base_url="https://api.smartystreets.com")
    def test_one_address(self, smarty_client, respx_mock):
        """Ensure singluar street address method returns an Address"""
        respx_mock.post("/street-address", data="").mock(
            return_value=httpx.Response(200, json=[{"street_address": "100 Main St"}]),
        )
        response = smarty_client.street_address({"street": "100 Main st"})
        assert isinstance(response, data.Address)

    def test_addresses_response(self, smarty_client, respx_mock):
        """Ensure address return an AddressCollection"""
        respx_mock.post("/street-address", data="").mock(
            return_value=httpx.Response(
                200,
                json=[
                    {"street_address": "100 Main St"},
                    {"street_address": "200 Main St"},
                ],
            )
        ),
        response = smarty_client.street_addresses(
            [{"street": "100 Main st"}, {"street": "200 Main St"}]
        )
        assert isinstance(response, data.AddressCollection)
        assert len(response) == 2
